/*  This file is part of SAIL (https://github.com/smoked-herring/sail)

    Copyright (c) 2020 Dmitry Baryshev

    The MIT License

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
*/

#include "sail-common.h"

#include "munit.h"

/*
 * Pixel formats.
 */
static MunitResult test_pixel_format_to_string(const MunitParameter params[], void *user_data) {
    (void)params;
    (void)user_data;

    const char *result;

#define TEST_SAIL_CONVERSION(e, s)           \
    result = NULL;                           \
    sail_pixel_format_to_string(e, &result); \
    munit_assert_string_equal(result, s);

    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_UNKNOWN, "UNKNOWN");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_SOURCE,  "SOURCE");

    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP1_INDEXED,  "BPP1-INDEXED");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP2_INDEXED,  "BPP2-INDEXED");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP4_INDEXED,  "BPP4-INDEXED");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP8_INDEXED,  "BPP8-INDEXED");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP16_INDEXED, "BPP16-INDEXED");

    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP1_GRAYSCALE,  "BPP1-GRAYSCALE");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP2_GRAYSCALE,  "BPP2-GRAYSCALE");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP4_GRAYSCALE,  "BPP4-GRAYSCALE");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP8_GRAYSCALE,  "BPP8-GRAYSCALE");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP16_GRAYSCALE, "BPP16-GRAYSCALE");

    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP4_GRAYSCALE_ALPHA,  "BPP4-GRAYSCALE-ALPHA");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP8_GRAYSCALE_ALPHA,  "BPP8-GRAYSCALE-ALPHA");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP16_GRAYSCALE_ALPHA, "BPP16-GRAYSCALE-ALPHA");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP32_GRAYSCALE_ALPHA, "BPP32-GRAYSCALE-ALPHA");

    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP16_RGB555, "BPP16-RGB555");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP16_BGR555, "BPP16-BGR555");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP16_RGB565, "BPP16-RGB565");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP16_BGR565, "BPP16-BGR565");

    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP24_RGB, "BPP24-RGB");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP24_BGR, "BPP24-BGR");

    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP48_RGB, "BPP48-RGB");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP48_BGR, "BPP48-BGR");

    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP32_RGBX, "BPP32-RGBX");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP32_BGRX, "BPP32-BGRX");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP32_XRGB, "BPP32-XRGB");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP32_XBGR, "BPP32-XBGR");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP32_RGBA, "BPP32-RGBA");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP32_BGRA, "BPP32-BGRA");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP32_ARGB, "BPP32-ARGB");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP32_ABGR, "BPP32-ABGR");

    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP64_RGBX, "BPP64-RGBX");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP64_BGRX, "BPP64-BGRX");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP64_XRGB, "BPP64-XRGB");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP64_XBGR, "BPP64-XBGR");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP64_RGBA, "BPP64-RGBA");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP64_BGRA, "BPP64-BGRA");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP64_ARGB, "BPP64-ARGB");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP64_ABGR, "BPP64-ABGR");

    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP32_CMYK, "BPP32-CMYK");
    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP64_CMYK, "BPP64-CMYK");

    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP24_YCBCR, "BPP24-YCBCR");

    TEST_SAIL_CONVERSION(SAIL_PIXEL_FORMAT_BPP32_YCCK, "BPP32-YCCK");

#undef TEST_SAIL_CONVERSION

    return MUNIT_OK;
}

static MunitResult test_pixel_format_from_string(const MunitParameter params[], void *user_data) {
    (void)params;
    (void)user_data;

    enum SailPixelFormat result;

#define TEST_SAIL_CONVERSION(s, e)             \
    result = SAIL_PIXEL_FORMAT_UNKNOWN;        \
    sail_pixel_format_from_string(s, &result); \
    munit_assert(result == e);

    TEST_SAIL_CONVERSION("UNKNOWN", SAIL_PIXEL_FORMAT_UNKNOWN);
    TEST_SAIL_CONVERSION("SOURCE",  SAIL_PIXEL_FORMAT_SOURCE);

    TEST_SAIL_CONVERSION("BPP1-INDEXED",  SAIL_PIXEL_FORMAT_BPP1_INDEXED);
    TEST_SAIL_CONVERSION("BPP2-INDEXED",  SAIL_PIXEL_FORMAT_BPP2_INDEXED);
    TEST_SAIL_CONVERSION("BPP4-INDEXED",  SAIL_PIXEL_FORMAT_BPP4_INDEXED);
    TEST_SAIL_CONVERSION("BPP8-INDEXED",  SAIL_PIXEL_FORMAT_BPP8_INDEXED);
    TEST_SAIL_CONVERSION("BPP16-INDEXED", SAIL_PIXEL_FORMAT_BPP16_INDEXED);

    TEST_SAIL_CONVERSION("BPP1-GRAYSCALE",  SAIL_PIXEL_FORMAT_BPP1_GRAYSCALE);
    TEST_SAIL_CONVERSION("BPP2-GRAYSCALE",  SAIL_PIXEL_FORMAT_BPP2_GRAYSCALE);
    TEST_SAIL_CONVERSION("BPP4-GRAYSCALE",  SAIL_PIXEL_FORMAT_BPP4_GRAYSCALE);
    TEST_SAIL_CONVERSION("BPP8-GRAYSCALE",  SAIL_PIXEL_FORMAT_BPP8_GRAYSCALE);
    TEST_SAIL_CONVERSION("BPP16-GRAYSCALE", SAIL_PIXEL_FORMAT_BPP16_GRAYSCALE);

    TEST_SAIL_CONVERSION("BPP4-GRAYSCALE-ALPHA",  SAIL_PIXEL_FORMAT_BPP4_GRAYSCALE_ALPHA);
    TEST_SAIL_CONVERSION("BPP8-GRAYSCALE-ALPHA",  SAIL_PIXEL_FORMAT_BPP8_GRAYSCALE_ALPHA);
    TEST_SAIL_CONVERSION("BPP16-GRAYSCALE-ALPHA", SAIL_PIXEL_FORMAT_BPP16_GRAYSCALE_ALPHA);
    TEST_SAIL_CONVERSION("BPP32-GRAYSCALE-ALPHA", SAIL_PIXEL_FORMAT_BPP32_GRAYSCALE_ALPHA);

    TEST_SAIL_CONVERSION("BPP16-RGB555", SAIL_PIXEL_FORMAT_BPP16_RGB555);
    TEST_SAIL_CONVERSION("BPP16-BGR555", SAIL_PIXEL_FORMAT_BPP16_BGR555);
    TEST_SAIL_CONVERSION("BPP16-RGB565", SAIL_PIXEL_FORMAT_BPP16_RGB565);
    TEST_SAIL_CONVERSION("BPP16-BGR565", SAIL_PIXEL_FORMAT_BPP16_BGR565);

    TEST_SAIL_CONVERSION("BPP24-RGB", SAIL_PIXEL_FORMAT_BPP24_RGB);
    TEST_SAIL_CONVERSION("BPP24-BGR", SAIL_PIXEL_FORMAT_BPP24_BGR);

    TEST_SAIL_CONVERSION("BPP48-RGB", SAIL_PIXEL_FORMAT_BPP48_RGB);
    TEST_SAIL_CONVERSION("BPP48-BGR", SAIL_PIXEL_FORMAT_BPP48_BGR);

    TEST_SAIL_CONVERSION("BPP32-RGBX", SAIL_PIXEL_FORMAT_BPP32_RGBX);
    TEST_SAIL_CONVERSION("BPP32-BGRX", SAIL_PIXEL_FORMAT_BPP32_BGRX);
    TEST_SAIL_CONVERSION("BPP32-XRGB", SAIL_PIXEL_FORMAT_BPP32_XRGB);
    TEST_SAIL_CONVERSION("BPP32-XBGR", SAIL_PIXEL_FORMAT_BPP32_XBGR);
    TEST_SAIL_CONVERSION("BPP32-RGBA", SAIL_PIXEL_FORMAT_BPP32_RGBA);
    TEST_SAIL_CONVERSION("BPP32-BGRA", SAIL_PIXEL_FORMAT_BPP32_BGRA);
    TEST_SAIL_CONVERSION("BPP32-ARGB", SAIL_PIXEL_FORMAT_BPP32_ARGB);
    TEST_SAIL_CONVERSION("BPP32-ABGR", SAIL_PIXEL_FORMAT_BPP32_ABGR);

    TEST_SAIL_CONVERSION("BPP64-RGBX", SAIL_PIXEL_FORMAT_BPP64_RGBX);
    TEST_SAIL_CONVERSION("BPP64-BGRX", SAIL_PIXEL_FORMAT_BPP64_BGRX);
    TEST_SAIL_CONVERSION("BPP64-XRGB", SAIL_PIXEL_FORMAT_BPP64_XRGB);
    TEST_SAIL_CONVERSION("BPP64-XBGR", SAIL_PIXEL_FORMAT_BPP64_XBGR);
    TEST_SAIL_CONVERSION("BPP64-RGBA", SAIL_PIXEL_FORMAT_BPP64_RGBA);
    TEST_SAIL_CONVERSION("BPP64-BGRA", SAIL_PIXEL_FORMAT_BPP64_BGRA);
    TEST_SAIL_CONVERSION("BPP64-ARGB", SAIL_PIXEL_FORMAT_BPP64_ARGB);
    TEST_SAIL_CONVERSION("BPP64-ABGR", SAIL_PIXEL_FORMAT_BPP64_ABGR);

    TEST_SAIL_CONVERSION("BPP32-CMYK", SAIL_PIXEL_FORMAT_BPP32_CMYK);
    TEST_SAIL_CONVERSION("BPP64-CMYK", SAIL_PIXEL_FORMAT_BPP64_CMYK);

    TEST_SAIL_CONVERSION("BPP24-YCBCR", SAIL_PIXEL_FORMAT_BPP24_YCBCR);

    TEST_SAIL_CONVERSION("BPP32-YCCK", SAIL_PIXEL_FORMAT_BPP32_YCCK);

#undef TEST_SAIL_CONVERSION

    return MUNIT_OK;
}

/*
 * Image properties.
 */
static MunitResult test_image_property_to_string(const MunitParameter params[], void *user_data) {
    (void)params;
    (void)user_data;

    const char *result;

#define TEST_SAIL_CONVERSION(e, s)             \
    sail_image_property_to_string(e, &result); \
    munit_assert_string_equal(result, s);

    TEST_SAIL_CONVERSION(SAIL_IMAGE_PROPERTY_FLIPPED_VERTICALLY, "FLIPPED-VERTICALLY");
    TEST_SAIL_CONVERSION(SAIL_IMAGE_PROPERTY_INTERLACED,         "INTERLACED");

#undef TEST_SAIL_CONVERSION

    return MUNIT_OK;
}

static MunitResult test_image_property_from_string(const MunitParameter params[], void *user_data) {
    (void)params;
    (void)user_data;

    enum SailImageProperties result;

#define TEST_SAIL_CONVERSION(s, e)               \
    sail_image_property_from_string(s, &result); \
    munit_assert(result == e);

    TEST_SAIL_CONVERSION("FLIPPED-VERTICALLY", SAIL_IMAGE_PROPERTY_FLIPPED_VERTICALLY);
    TEST_SAIL_CONVERSION("INTERLACED",         SAIL_IMAGE_PROPERTY_INTERLACED);

#undef TEST_SAIL_CONVERSION

    return MUNIT_OK;
}

/*
 * Compression types.
 */
static MunitResult test_compression_type_to_string(const MunitParameter params[], void *user_data) {
    (void)params;
    (void)user_data;

    const char *result;

#define TEST_SAIL_CONVERSION(e, s)               \
    result = NULL;                               \
    sail_compression_type_to_string(e, &result); \
    munit_assert_string_equal(result, s);

    TEST_SAIL_CONVERSION(SAIL_COMPRESSION_UNSUPPORTED, "UNSUPPORTED");
    TEST_SAIL_CONVERSION(SAIL_COMPRESSION_NO,          "NO");
    TEST_SAIL_CONVERSION(SAIL_COMPRESSION_RLE,         "RLE");

#undef TEST_SAIL_CONVERSION

    return MUNIT_OK;
}

static MunitResult test_compression_type_from_string(const MunitParameter params[], void *user_data) {
    (void)params;
    (void)user_data;

    enum SailCompressionType result;

#define TEST_SAIL_CONVERSION(s, e)                 \
    result = SAIL_COMPRESSION_UNSUPPORTED;         \
    sail_compression_type_from_string(s, &result); \
    munit_assert(result == e);

    TEST_SAIL_CONVERSION("UNSUPPORTED", SAIL_COMPRESSION_UNSUPPORTED);
    TEST_SAIL_CONVERSION("NO",          SAIL_COMPRESSION_NO);
    TEST_SAIL_CONVERSION("RLE",         SAIL_COMPRESSION_RLE);

#undef TEST_SAIL_CONVERSION

    return MUNIT_OK;
}

/*
 * Plugin features.
 */
static MunitResult test_plugin_feature_to_string(const MunitParameter params[], void *user_data) {
    (void)params;
    (void)user_data;

    const char *result;

#define TEST_SAIL_CONVERSION(e, s)             \
    sail_plugin_feature_to_string(e, &result); \
    munit_assert_string_equal(result, s);

    TEST_SAIL_CONVERSION(SAIL_PLUGIN_FEATURE_STATIC,     "STATIC");
    TEST_SAIL_CONVERSION(SAIL_PLUGIN_FEATURE_ANIMATED,   "ANIMATED");
    TEST_SAIL_CONVERSION(SAIL_PLUGIN_FEATURE_MULTIPAGED, "MULTIPAGED");
    TEST_SAIL_CONVERSION(SAIL_PLUGIN_FEATURE_META_INFO,  "META-INFO");
    TEST_SAIL_CONVERSION(SAIL_PLUGIN_FEATURE_EXIF,       "EXIF");
    TEST_SAIL_CONVERSION(SAIL_PLUGIN_FEATURE_INTERLACED, "INTERLACED");
    TEST_SAIL_CONVERSION(SAIL_PLUGIN_FEATURE_ICCP,       "ICCP");

#undef TEST_SAIL_CONVERSION

    return MUNIT_OK;
}

static MunitResult test_plugin_feature_from_string(const MunitParameter params[], void *user_data) {
    (void)params;
    (void)user_data;

    enum SailPluginFeatures result;

#define TEST_SAIL_CONVERSION(s, e)               \
    sail_plugin_feature_from_string(s, &result); \
    munit_assert(result == e);

    TEST_SAIL_CONVERSION("STATIC",     SAIL_PLUGIN_FEATURE_STATIC);
    TEST_SAIL_CONVERSION("ANIMATED",   SAIL_PLUGIN_FEATURE_ANIMATED);
    TEST_SAIL_CONVERSION("MULTIPAGED", SAIL_PLUGIN_FEATURE_MULTIPAGED);
    TEST_SAIL_CONVERSION("META-INFO",  SAIL_PLUGIN_FEATURE_META_INFO);
    TEST_SAIL_CONVERSION("EXIF",       SAIL_PLUGIN_FEATURE_EXIF);
    TEST_SAIL_CONVERSION("INTERLACED", SAIL_PLUGIN_FEATURE_INTERLACED);
    TEST_SAIL_CONVERSION("ICCP",       SAIL_PLUGIN_FEATURE_ICCP);

#undef TEST_SAIL_CONVERSION

    return MUNIT_OK;
}

static MunitTest test_suite_tests[] = {
    { (char *)"/pixel-format-to-string",   test_pixel_format_to_string,     NULL, NULL, MUNIT_TEST_OPTION_NONE, NULL },
    { (char *)"/pixel-format-from-string", test_pixel_format_from_string,   NULL, NULL, MUNIT_TEST_OPTION_NONE, NULL },

    { (char *)"/image-property-to-string",   test_image_property_to_string,   NULL, NULL, MUNIT_TEST_OPTION_NONE, NULL },
    { (char *)"/image-property-from-string", test_image_property_from_string, NULL, NULL, MUNIT_TEST_OPTION_NONE, NULL },

    { (char *)"/compression-type-to-string",   test_compression_type_to_string,   NULL, NULL, MUNIT_TEST_OPTION_NONE, NULL },
    { (char *)"/compression-type-from-string", test_compression_type_from_string, NULL, NULL, MUNIT_TEST_OPTION_NONE, NULL },

    { (char *)"/plugin-feature-to-string",   test_plugin_feature_to_string,   NULL, NULL, MUNIT_TEST_OPTION_NONE, NULL },
    { (char *)"/plugin-feature-from-string", test_plugin_feature_from_string, NULL, NULL, MUNIT_TEST_OPTION_NONE, NULL },

    { NULL, NULL, NULL, NULL, MUNIT_TEST_OPTION_NONE, NULL }
};

static const MunitSuite test_suite = {
    (char *)"/integrity",
    test_suite_tests,
    NULL,
    1,
    MUNIT_SUITE_OPTION_NONE
};

int main(int argc, char *argv[MUNIT_ARRAY_PARAM(argc + 1)]) {
    return munit_suite_main(&test_suite, NULL, argc, argv);
}
